@@ -8,7 +8,8 @@ import pytest
 import xonsh
 from xonsh.platform import ON_WINDOWS
 
-from tools import skip_if_on_windows
+from tools import (skip_if_on_windows, skip_if_on_darwin, skip_if_on_travis,
+    ON_WINDOWS, ON_DARWIN, ON_TRAVIS)
 
 
 XONSH_PREFIX = xonsh.__file__
@@ -28,6 +29,10 @@ PATH = os.path.join(os.path.dirname(__file__), 'bin') + os.pathsep + \
        os.environ['PATH']
 
 
+skip_if_no_xonsh = pytest.mark.skipif(shutil.which('xonsh', path=PATH) is None,
+                                      reason='xonsh not on path')
+
+
 def run_xonsh(cmd, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.STDOUT):
     env = dict(os.environ)
     env['PATH'] = PATH
@@ -51,6 +56,20 @@ def run_xonsh(cmd, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.STDOUT):
         raise
     return out, err, proc.returncode
 
+
+def check_run_xonsh(cmd, fmt, exp):
+    """The ``fmt`` parameter is a function
+    that formats the output of cmd, can be None.
+    """
+    out, err, rtn = run_xonsh(cmd, stderr=sp.DEVNULL)
+    if callable(fmt):
+        out = fmt(out)
+    if callable(exp):
+        exp = exp()
+    assert out == exp
+    assert rtn == 0
+
+
 #
 # The following list contains a (stdin, stdout, returncode) tuples
 #
@@ -146,6 +165,58 @@ with open('tttt', 'w') as fp:
 
 ![cat tttt | wc | wc]
 """, '      1       3      24\n' if ON_WINDOWS else " 1  4 16 <stdin>\n", 0),
+# test unthreadable alias (which should trigger a ProcPoxy call)
+("""
+from xonsh.tools import unthreadable
+
+@unthreadable
+def _f():
+    return 'hello\\n'
+
+aliases['f'] = _f
+f
+""", "hello\n", 0),
+# test ambiguous globs
+("""
+import os
+
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+
+files = ['Actually_test.tst', 'Actually.tst', 'Complete_test.tst', 'Complete.tst']
+
+# touch the file
+for f in files:
+    with open(f, 'w'):
+        pass
+
+# echo the files
+echo *.tst and echo *_test.tst
+echo *_test.tst
+echo *_test.tst and echo *.tst
+
+# remove the files
+for f in files:
+    os.remove(f)
+""",
+'Actually.tst Actually_test.tst Complete.tst Complete_test.tst\n'
+'Actually_test.tst Complete_test.tst\n'
+'Actually_test.tst Complete_test.tst\n'
+'Actually_test.tst Complete_test.tst\n'
+'Actually.tst Actually_test.tst Complete.tst Complete_test.tst\n',
+0),
+#
+# test ambiguous line continuations
+#
+("""
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+
+echo --option1 \
+--option2
+""", '--option1 --option2\n', 0),
 ]
 
 
@@ -175,24 +246,53 @@ def test_script_stder(case):
     assert exp_err == err
     assert exp_rtn == rtn
 
-
 @skip_if_on_windows
 @pytest.mark.parametrize('cmd, fmt, exp', [
     ('pwd', None, lambda: os.getcwd() + '\n'),
     ('echo WORKING', None, 'WORKING\n'),
     ('ls -f', lambda out: out.splitlines().sort(), os.listdir().sort()),
     ])
-def test_single_command(cmd, fmt, exp):
-    """The ``fmt`` parameter is a function
-    that formats the output of cmd, can be None.
-    """
-    out, err, rtn = run_xonsh(cmd, stderr=sp.DEVNULL)
-    if callable(fmt):
-        out = fmt(out)
-    if callable(exp):
-        exp = exp()
-    assert out == exp
-    assert rtn == 0
+def test_single_command_no_windows(cmd, fmt, exp):
+    check_run_xonsh(cmd, fmt, exp)
+
+
+_bad_case = pytest.mark.skipif(ON_DARWIN or ON_WINDOWS or ON_TRAVIS,
+                               reason="bad platforms")
+
+@_bad_case
+def test_printfile():
+    check_run_xonsh('printfile.xsh', None, 'printfile.xsh\n')
+
+
+@_bad_case
+def test_printname():
+    check_run_xonsh('printfile.xsh', None, 'printfile.xsh\n')
+
+
+@_bad_case
+def test_sourcefile():
+    check_run_xonsh('printfile.xsh', None, 'printfile.xsh\n')
+
+
+@_bad_case
+@pytest.mark.parametrize('cmd, fmt, exp', [
+    # test subshell wrapping
+    ("""
+with open('tttt', 'w') as fp:
+    fp.write("Wow mom!\\n")
+
+(wc) < tttt
+""", None, " 1  2 9 <stdin>\n"),
+    # test subshell statement wrapping
+    ("""
+with open('tttt', 'w') as fp:
+    fp.write("Wow mom!\\n")
+
+(wc;) < tttt
+""", None, " 1  2 9 <stdin>\n"),
+])
+def test_subshells(cmd, fmt, exp):
+    check_run_xonsh(cmd, fmt, exp)
 
 
 @skip_if_on_windows
