@@ -23,7 +23,8 @@ from xonsh.tools import (
     to_dynamic_cwd_tuple, to_logfile_opt, pathsep_to_set, set_to_pathsep,
     is_string_seq, pathsep_to_seq, seq_to_pathsep, is_nonstring_seq_of_strings,
     pathsep_to_upper_seq, seq_to_upper_pathsep, expandvars, is_int_as_str, is_slice_as_str,
-    ensure_timestamp, get_portions
+    ensure_timestamp, get_portions, is_balanced, subexpr_before_unbalanced,
+    swap_values, get_logical_line, replace_logical_line, check_quotes,
     )
 from xonsh.environ import Env
 
@@ -207,6 +208,33 @@ def test_subproc_toks_hello_mom_second():
     assert exp == obs
 
 
+def test_subproc_toks_hello_bad_leading_single_quotes():
+    obs = subproc_toks('echo "hello', lexer=LEXER, returnline=True)
+    assert obs is None
+
+
+def test_subproc_toks_hello_bad_trailing_single_quotes():
+    obs = subproc_toks('echo hello"', lexer=LEXER, returnline=True)
+    assert obs is None
+
+
+def test_subproc_toks_hello_bad_leading_triple_quotes():
+    obs = subproc_toks('echo """hello', lexer=LEXER, returnline=True)
+    assert obs is None
+
+
+def test_subproc_toks_hello_bad_trailing_triple_quotes():
+    obs = subproc_toks('echo hello"""', lexer=LEXER, returnline=True)
+    assert obs is None
+
+
+def test_subproc_toks_hello_mom_triple_quotes_nl():
+    s = 'echo """hello\nmom"""'
+    exp = '![{0}]'.format(s)
+    obs = subproc_toks(s, lexer=LEXER, returnline=True)
+    assert exp == obs
+
+
 def test_subproc_toks_comment():
     exp = None
     obs = subproc_toks('# I am a comment', lexer=LEXER, returnline=True)
@@ -322,6 +350,110 @@ def test_subproc_toks_pyeval_redirect():
     assert exp == obs
 
 
+def test_subproc_toks_greedy_parens():
+    s = '(sort)'
+    exp = '![{0}]'.format(s)
+    obs = subproc_toks(s, lexer=LEXER, returnline=True, greedy=True)
+    assert exp == obs
+
+
+def test_subproc_toks_greedy_parens_inp():
+    s = '(sort) < input.txt'
+    exp = '![{0}]'.format(s)
+    obs = subproc_toks(s, lexer=LEXER, returnline=True, greedy=True)
+    assert exp == obs
+
+
+def test_subproc_toks_greedy_parens_statements():
+    s = '(echo "abc"; sleep 1; echo "def")'
+    exp = '![{0}]'.format(s)
+    obs = subproc_toks(s, lexer=LEXER, returnline=True, greedy=True)
+    assert exp == obs
+
+
+def test_subproc_toks_greedy_parens_statements_with_grep():
+    s = '(echo "abc"; sleep 1; echo "def") | grep'
+    exp = '![{0}]'.format(s)
+    obs = subproc_toks(s, lexer=LEXER, returnline=True, greedy=True)
+    assert exp == obs
+
+
+LOGICAL_LINE_CASES = [
+("""x = 14 + 2""", 0, 'x = 14 + 2', 1),
+("""x = \\
+14 \\
++ 2
+""", 0, 'x = 14 + 2', 3),
+("""y = 16
+14 \\
++ 2
+""", 1, '14 + 2', 2),
+('''x = """wow
+mom"""
+''', 0, 'x = """wow\nmom"""', 2),
+# test from start
+("echo --option1 value1 \\\n"
+ "     --option2 value2 \\\n"
+ "     --optionZ valueZ",
+0, "echo --option1 value1      --option2 value2      --optionZ valueZ", 3),
+# test from second line
+("echo --option1 value1 \\\n"
+ "     --option2 value2 \\\n"
+ "     --optionZ valueZ",
+1, "echo --option1 value1      --option2 value2      --optionZ valueZ", 3),
+]
+
+@pytest.mark.parametrize('src, idx, exp_line, exp_n', LOGICAL_LINE_CASES)
+def test_get_logical_line(src, idx, exp_line, exp_n):
+    lines = src.splitlines()
+    line, n, start = get_logical_line(lines, idx)
+    assert exp_line == line
+    assert exp_n == n
+
+
+@pytest.mark.parametrize('src, idx, exp_line, exp_n', LOGICAL_LINE_CASES)
+def test_replace_logical_line(src, idx, exp_line, exp_n):
+    lines = src.splitlines()
+    logical = exp_line
+    while idx > 0 and lines[idx-1].endswith('\\'):
+        idx -= 1
+    replace_logical_line(lines, logical, idx, exp_n)
+    exp = src.replace('\\\n', '').strip()
+    obs = '\n'.join(lines).replace('\\\n', '').strip()
+    assert exp == obs
+
+
+@pytest.mark.parametrize('inp, exp', [
+    ('f(1,10),x.y', True),
+    ('"x"', True),
+    ("'y'", True),
+    ('b"x"', True),
+    ("r'y'", True),
+    ('"""hello\nmom"""', True),
+])
+def test_check_quotes(inp, exp):
+    obs = check_quotes(inp)
+    assert exp is obs
+
+
+@pytest.mark.parametrize('inp', [
+    'f(1,10),x.y',
+])
+def test_is_balanced_parens(inp):
+    obs = is_balanced(inp, '(', ')')
+    assert obs
+
+
+@pytest.mark.parametrize('inp', [
+    'f(x.',
+    'f(1,x.'
+    'f((1,10),x.y',
+])
+def test_is_not_balanced_parens(inp):
+    obs = is_balanced(inp, '(', ')')
+    assert not obs
+
+
 @pytest.mark.parametrize('inp, exp', [
     ('f(x.', 'x.'),
     ('f(1,x.', 'x.'),
@@ -332,6 +464,20 @@ def test_subexpr_from_unbalanced_parens(inp, exp):
     assert exp == obs
 
 
+@pytest.mark.parametrize('inp, exp', [
+    ('f(x.', 'f'),
+    ('f(1,x.', 'f'),
+    ('f((1,10),x.y', 'f'),
+    ('wakka().f((1,10),x.y', '.f'),
+    ('wakka(f((1,10),x.y', 'f'),
+    ('wakka(jawakka().f((1,10),x.y', '.f'),
+    ('wakka(jawakka().f((1,10),x.y)', 'wakka'),
+])
+def test_subexpr_before_unbalanced_parens(inp, exp):
+    obs = subexpr_before_unbalanced(inp, '(', ')')
+    assert exp == obs
+
+
 @pytest.mark.parametrize('line, mincol, exp', [
     ('ls && echo a', 0, 4),
     ('ls && echo a', 6, None),
@@ -1096,3 +1242,13 @@ def test_expand_path(expand_user, inp, expand_env_vars, exp_end, xonsh_builtins)
         assert path == home_path + exp_end
     else:
         assert path == '~' + exp_end
+
+
+def test_swap_values():
+    orig = {'x': 1}
+    updates = {'x': 42, 'y': 43}
+    with swap_values(orig, updates):
+        assert orig['x'] == 42
+        assert orig['y'] == 43
+    assert orig['x'] == 1
+    assert 'y' not in orig
